from numba import cuda
import numpy

def start_kernel(Cx, Cy, Dx, Dy, Dz, v, w):
    Cx_global = cuda.to_device(Cx)
    Cy_global = cuda.to_device(Cy)
    Dx_global = cuda.to_device(Dx)
    Dy_global = cuda.to_device(Dy)
    Dz_global = cuda.to_device(Dz)

    egyensulyi_mtx = cuda.device_array((Dz.size, 4, 4))

    gpu_egyensuly[256, 256](v, w, Cx_global, Cy_global, Dx_global, Dy_global, Dz_global, egyensulyi_mtx)

    return egyensulyi_mtx.copy_to_host()


@cuda.jit
def gpu_egyensuly(v, w, Cx, Cy, Dx, Dy, Dz, egyensulyi_mtx):
    ABC_oldal(v, w, Cx, Cy, Dx, Dy, Dz, egyensulyi_mtx)


@cuda.jit(device=True)
def ABC_oldal(v, w, Cx_arr, Cy_arr, Dx_arr, Dy_arr, Dz_arr, egyensulyi_mtx):
    pos = cuda.grid(1)
    if pos >= Dx_arr.size:
        return
    Cx = Cx_arr[pos]
    Cy = Cy_arr[pos]
    Dx = Dx_arr[pos]
    Dy = Dy_arr[pos]
    Dz = Dz_arr[pos]

    ABx = 1.0/v
    ABy = 0.0
    ABz = 0.0

    ACx = (Cx - 0.0)/v
    ACy = (Cy - 0.0)/v
    ACz = 0.0            # (0.0 - 0.0)/v

    for i in range(v+1):
        for j in range(v+1):
            oKx = i * ABx + j * ACx
            oKy = i * ABy + j * ACy
            oKz = i * ABz + j * ACz
            Lx = (Dx - oKx)/w
            Ly = (Dy - oKy)/w
            Lz = (Dz - oKz)/w

            for k in range(w+1):
                Sx = oKx + Lx*k
                Sy = oKy + Ly*k
                Sz = oKz + Lz*k

                S = stabil_ep(Sx, Sy, Sz, Cx, Cy, Dx, Dy, Dz)
                U = instabil_ep(Sx, Sy, Sz, Cx, Cy, Dx, Dy, Dz)
                H = nyereg_ep(Sx, Sy, Sz, Cx, Cy, Dx, Dy, Dz)

                if S + U - H == 2:
                    egyensulyi_mtx[pos, S, U] = 1




# Sxyz - a skp pontja, (0,0,0), (0,0,1), C(x,y,0), D(x,y,z) a tetraéderhez tartozó csúcspontok
# return stabil sp száma S súlypontnál
@cuda.jit(device=True)
def stabil_ep(Sx, Sy, Sz, Cx, Cy, Dx, Dy, Dz):
    return 0

# Sxyz - a skp pontja, (0,0,0), (0,0,1), C(x,y,0), D(x,y,z) a tetraéderhez tartozó csúcspontok
# return instabil sp száma S a testre nézve
@cuda.jit(device=True)
def instabil_ep(Sx, Sy, Sz, Cx, Cy, Dx, Dy, Dz):
    return 0

# Sxyz - a skp pontja, (0,0,0), (0,0,1), C(x,y,0), D(x,y,z) a tetraéderhez tartozó csúcspontok
# return nyereg sp száma S az IJKL testre nézve
@cuda.jit(device=True)
def nyereg_ep(Sx, Sy, Sz, Cx, Cy, Dx, Dy, Dz):
    return 0